# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorboard/uploader/proto/write_service.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tensorboard.uploader.proto import scalar_pb2 as tensorboard_dot_uploader_dot_proto_dot_scalar__pb2
from tensorboard.compat.proto import summary_pb2 as tensorboard_dot_compat_dot_proto_dot_summary__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='tensorboard/uploader/proto/write_service.proto',
  package='tensorboard.service',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n.tensorboard/uploader/proto/write_service.proto\x12\x13tensorboard.service\x1a\'tensorboard/uploader/proto/scalar.proto\x1a&tensorboard/compat/proto/summary.proto\"\x19\n\x17\x43reateExperimentRequest\">\n\x18\x43reateExperimentResponse\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\x12\x0b\n\x03url\x18\x02 \x01(\t\"0\n\x17\x44\x65leteExperimentRequest\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\"\x1a\n\x18\x44\x65leteExperimentResponse\"\'\n\x10PurgeDataRequest\x12\x13\n\x0b\x62\x61tch_limit\x18\x01 \x01(\x05\"I\n\x11PurgeDataResponse\x12\x34\n\x0bpurge_stats\x18\x01 \x01(\x0b\x32\x1f.tensorboard.service.PurgeStats\">\n\nPurgeStats\x12\x0c\n\x04tags\x18\x01 \x01(\x05\x12\x13\n\x0b\x65xperiments\x18\x02 \x01(\x05\x12\r\n\x05users\x18\x03 \x01(\x05\"\xad\x02\n\x12WriteScalarRequest\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\x12\x39\n\x04runs\x18\x02 \x03(\x0b\x32+.tensorboard.service.WriteScalarRequest.Run\x1aN\n\x03Run\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x39\n\x04tags\x18\x02 \x03(\x0b\x32+.tensorboard.service.WriteScalarRequest.Tag\x1au\n\x03Tag\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x30\n\x06points\x18\x02 \x03(\x0b\x32 .tensorboard.service.ScalarPoint\x12.\n\x08metadata\x18\x03 \x01(\x0b\x32\x1c.tensorboard.SummaryMetadata\"\x15\n\x13WriteScalarResponse\"\x16\n\x14\x44\x65leteOwnUserRequest\"\x17\n\x15\x44\x65leteOwnUserResponse2\xac\x04\n\x18TensorBoardWriterService\x12q\n\x10\x43reateExperiment\x12,.tensorboard.service.CreateExperimentRequest\x1a-.tensorboard.service.CreateExperimentResponse\"\x00\x12q\n\x10\x44\x65leteExperiment\x12,.tensorboard.service.DeleteExperimentRequest\x1a-.tensorboard.service.DeleteExperimentResponse\"\x00\x12\\\n\tPurgeData\x12%.tensorboard.service.PurgeDataRequest\x1a&.tensorboard.service.PurgeDataResponse\"\x00\x12\x62\n\x0bWriteScalar\x12\'.tensorboard.service.WriteScalarRequest\x1a(.tensorboard.service.WriteScalarResponse\"\x00\x12h\n\rDeleteOwnUser\x12).tensorboard.service.DeleteOwnUserRequest\x1a*.tensorboard.service.DeleteOwnUserResponse\"\x00\x62\x06proto3')
  ,
  dependencies=[tensorboard_dot_uploader_dot_proto_dot_scalar__pb2.DESCRIPTOR,tensorboard_dot_compat_dot_proto_dot_summary__pb2.DESCRIPTOR,])




_CREATEEXPERIMENTREQUEST = _descriptor.Descriptor(
  name='CreateExperimentRequest',
  full_name='tensorboard.service.CreateExperimentRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=152,
  serialized_end=177,
)


_CREATEEXPERIMENTRESPONSE = _descriptor.Descriptor(
  name='CreateExperimentResponse',
  full_name='tensorboard.service.CreateExperimentResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='experiment_id', full_name='tensorboard.service.CreateExperimentResponse.experiment_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='url', full_name='tensorboard.service.CreateExperimentResponse.url', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=179,
  serialized_end=241,
)


_DELETEEXPERIMENTREQUEST = _descriptor.Descriptor(
  name='DeleteExperimentRequest',
  full_name='tensorboard.service.DeleteExperimentRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='experiment_id', full_name='tensorboard.service.DeleteExperimentRequest.experiment_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=243,
  serialized_end=291,
)


_DELETEEXPERIMENTRESPONSE = _descriptor.Descriptor(
  name='DeleteExperimentResponse',
  full_name='tensorboard.service.DeleteExperimentResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=293,
  serialized_end=319,
)


_PURGEDATAREQUEST = _descriptor.Descriptor(
  name='PurgeDataRequest',
  full_name='tensorboard.service.PurgeDataRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='batch_limit', full_name='tensorboard.service.PurgeDataRequest.batch_limit', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=321,
  serialized_end=360,
)


_PURGEDATARESPONSE = _descriptor.Descriptor(
  name='PurgeDataResponse',
  full_name='tensorboard.service.PurgeDataResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='purge_stats', full_name='tensorboard.service.PurgeDataResponse.purge_stats', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=362,
  serialized_end=435,
)


_PURGESTATS = _descriptor.Descriptor(
  name='PurgeStats',
  full_name='tensorboard.service.PurgeStats',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='tags', full_name='tensorboard.service.PurgeStats.tags', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='experiments', full_name='tensorboard.service.PurgeStats.experiments', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='users', full_name='tensorboard.service.PurgeStats.users', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=437,
  serialized_end=499,
)


_WRITESCALARREQUEST_RUN = _descriptor.Descriptor(
  name='Run',
  full_name='tensorboard.service.WriteScalarRequest.Run',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='tensorboard.service.WriteScalarRequest.Run.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tags', full_name='tensorboard.service.WriteScalarRequest.Run.tags', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=606,
  serialized_end=684,
)

_WRITESCALARREQUEST_TAG = _descriptor.Descriptor(
  name='Tag',
  full_name='tensorboard.service.WriteScalarRequest.Tag',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='tensorboard.service.WriteScalarRequest.Tag.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='points', full_name='tensorboard.service.WriteScalarRequest.Tag.points', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='metadata', full_name='tensorboard.service.WriteScalarRequest.Tag.metadata', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=686,
  serialized_end=803,
)

_WRITESCALARREQUEST = _descriptor.Descriptor(
  name='WriteScalarRequest',
  full_name='tensorboard.service.WriteScalarRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='experiment_id', full_name='tensorboard.service.WriteScalarRequest.experiment_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='runs', full_name='tensorboard.service.WriteScalarRequest.runs', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_WRITESCALARREQUEST_RUN, _WRITESCALARREQUEST_TAG, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=502,
  serialized_end=803,
)


_WRITESCALARRESPONSE = _descriptor.Descriptor(
  name='WriteScalarResponse',
  full_name='tensorboard.service.WriteScalarResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=805,
  serialized_end=826,
)


_DELETEOWNUSERREQUEST = _descriptor.Descriptor(
  name='DeleteOwnUserRequest',
  full_name='tensorboard.service.DeleteOwnUserRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=828,
  serialized_end=850,
)


_DELETEOWNUSERRESPONSE = _descriptor.Descriptor(
  name='DeleteOwnUserResponse',
  full_name='tensorboard.service.DeleteOwnUserResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=852,
  serialized_end=875,
)

_PURGEDATARESPONSE.fields_by_name['purge_stats'].message_type = _PURGESTATS
_WRITESCALARREQUEST_RUN.fields_by_name['tags'].message_type = _WRITESCALARREQUEST_TAG
_WRITESCALARREQUEST_RUN.containing_type = _WRITESCALARREQUEST
_WRITESCALARREQUEST_TAG.fields_by_name['points'].message_type = tensorboard_dot_uploader_dot_proto_dot_scalar__pb2._SCALARPOINT
_WRITESCALARREQUEST_TAG.fields_by_name['metadata'].message_type = tensorboard_dot_compat_dot_proto_dot_summary__pb2._SUMMARYMETADATA
_WRITESCALARREQUEST_TAG.containing_type = _WRITESCALARREQUEST
_WRITESCALARREQUEST.fields_by_name['runs'].message_type = _WRITESCALARREQUEST_RUN
DESCRIPTOR.message_types_by_name['CreateExperimentRequest'] = _CREATEEXPERIMENTREQUEST
DESCRIPTOR.message_types_by_name['CreateExperimentResponse'] = _CREATEEXPERIMENTRESPONSE
DESCRIPTOR.message_types_by_name['DeleteExperimentRequest'] = _DELETEEXPERIMENTREQUEST
DESCRIPTOR.message_types_by_name['DeleteExperimentResponse'] = _DELETEEXPERIMENTRESPONSE
DESCRIPTOR.message_types_by_name['PurgeDataRequest'] = _PURGEDATAREQUEST
DESCRIPTOR.message_types_by_name['PurgeDataResponse'] = _PURGEDATARESPONSE
DESCRIPTOR.message_types_by_name['PurgeStats'] = _PURGESTATS
DESCRIPTOR.message_types_by_name['WriteScalarRequest'] = _WRITESCALARREQUEST
DESCRIPTOR.message_types_by_name['WriteScalarResponse'] = _WRITESCALARRESPONSE
DESCRIPTOR.message_types_by_name['DeleteOwnUserRequest'] = _DELETEOWNUSERREQUEST
DESCRIPTOR.message_types_by_name['DeleteOwnUserResponse'] = _DELETEOWNUSERRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

CreateExperimentRequest = _reflection.GeneratedProtocolMessageType('CreateExperimentRequest', (_message.Message,), {
  'DESCRIPTOR' : _CREATEEXPERIMENTREQUEST,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.CreateExperimentRequest)
  })
_sym_db.RegisterMessage(CreateExperimentRequest)

CreateExperimentResponse = _reflection.GeneratedProtocolMessageType('CreateExperimentResponse', (_message.Message,), {
  'DESCRIPTOR' : _CREATEEXPERIMENTRESPONSE,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.CreateExperimentResponse)
  })
_sym_db.RegisterMessage(CreateExperimentResponse)

DeleteExperimentRequest = _reflection.GeneratedProtocolMessageType('DeleteExperimentRequest', (_message.Message,), {
  'DESCRIPTOR' : _DELETEEXPERIMENTREQUEST,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.DeleteExperimentRequest)
  })
_sym_db.RegisterMessage(DeleteExperimentRequest)

DeleteExperimentResponse = _reflection.GeneratedProtocolMessageType('DeleteExperimentResponse', (_message.Message,), {
  'DESCRIPTOR' : _DELETEEXPERIMENTRESPONSE,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.DeleteExperimentResponse)
  })
_sym_db.RegisterMessage(DeleteExperimentResponse)

PurgeDataRequest = _reflection.GeneratedProtocolMessageType('PurgeDataRequest', (_message.Message,), {
  'DESCRIPTOR' : _PURGEDATAREQUEST,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.PurgeDataRequest)
  })
_sym_db.RegisterMessage(PurgeDataRequest)

PurgeDataResponse = _reflection.GeneratedProtocolMessageType('PurgeDataResponse', (_message.Message,), {
  'DESCRIPTOR' : _PURGEDATARESPONSE,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.PurgeDataResponse)
  })
_sym_db.RegisterMessage(PurgeDataResponse)

PurgeStats = _reflection.GeneratedProtocolMessageType('PurgeStats', (_message.Message,), {
  'DESCRIPTOR' : _PURGESTATS,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.PurgeStats)
  })
_sym_db.RegisterMessage(PurgeStats)

WriteScalarRequest = _reflection.GeneratedProtocolMessageType('WriteScalarRequest', (_message.Message,), {

  'Run' : _reflection.GeneratedProtocolMessageType('Run', (_message.Message,), {
    'DESCRIPTOR' : _WRITESCALARREQUEST_RUN,
    '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.service.WriteScalarRequest.Run)
    })
  ,

  'Tag' : _reflection.GeneratedProtocolMessageType('Tag', (_message.Message,), {
    'DESCRIPTOR' : _WRITESCALARREQUEST_TAG,
    '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.service.WriteScalarRequest.Tag)
    })
  ,
  'DESCRIPTOR' : _WRITESCALARREQUEST,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.WriteScalarRequest)
  })
_sym_db.RegisterMessage(WriteScalarRequest)
_sym_db.RegisterMessage(WriteScalarRequest.Run)
_sym_db.RegisterMessage(WriteScalarRequest.Tag)

WriteScalarResponse = _reflection.GeneratedProtocolMessageType('WriteScalarResponse', (_message.Message,), {
  'DESCRIPTOR' : _WRITESCALARRESPONSE,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.WriteScalarResponse)
  })
_sym_db.RegisterMessage(WriteScalarResponse)

DeleteOwnUserRequest = _reflection.GeneratedProtocolMessageType('DeleteOwnUserRequest', (_message.Message,), {
  'DESCRIPTOR' : _DELETEOWNUSERREQUEST,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.DeleteOwnUserRequest)
  })
_sym_db.RegisterMessage(DeleteOwnUserRequest)

DeleteOwnUserResponse = _reflection.GeneratedProtocolMessageType('DeleteOwnUserResponse', (_message.Message,), {
  'DESCRIPTOR' : _DELETEOWNUSERRESPONSE,
  '__module__' : 'tensorboard.uploader.proto.write_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.DeleteOwnUserResponse)
  })
_sym_db.RegisterMessage(DeleteOwnUserResponse)



_TENSORBOARDWRITERSERVICE = _descriptor.ServiceDescriptor(
  name='TensorBoardWriterService',
  full_name='tensorboard.service.TensorBoardWriterService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=878,
  serialized_end=1434,
  methods=[
  _descriptor.MethodDescriptor(
    name='CreateExperiment',
    full_name='tensorboard.service.TensorBoardWriterService.CreateExperiment',
    index=0,
    containing_service=None,
    input_type=_CREATEEXPERIMENTREQUEST,
    output_type=_CREATEEXPERIMENTRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='DeleteExperiment',
    full_name='tensorboard.service.TensorBoardWriterService.DeleteExperiment',
    index=1,
    containing_service=None,
    input_type=_DELETEEXPERIMENTREQUEST,
    output_type=_DELETEEXPERIMENTRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='PurgeData',
    full_name='tensorboard.service.TensorBoardWriterService.PurgeData',
    index=2,
    containing_service=None,
    input_type=_PURGEDATAREQUEST,
    output_type=_PURGEDATARESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='WriteScalar',
    full_name='tensorboard.service.TensorBoardWriterService.WriteScalar',
    index=3,
    containing_service=None,
    input_type=_WRITESCALARREQUEST,
    output_type=_WRITESCALARRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='DeleteOwnUser',
    full_name='tensorboard.service.TensorBoardWriterService.DeleteOwnUser',
    index=4,
    containing_service=None,
    input_type=_DELETEOWNUSERREQUEST,
    output_type=_DELETEOWNUSERRESPONSE,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_TENSORBOARDWRITERSERVICE)

DESCRIPTOR.services_by_name['TensorBoardWriterService'] = _TENSORBOARDWRITERSERVICE

# @@protoc_insertion_point(module_scope)
